@@ -874,8 +874,8 @@ setMethod("getDesign", "unmarkedFrameOccuMS",
 
 })
 
-
 # pcountOpen
+#setMethod("getDesign", "unmarkedFramePCOorMMO",
 setMethod("getDesign", "unmarkedFramePCO",
     function(umf, formula, na.rm = TRUE)
 {
@@ -1019,6 +1019,158 @@ setMethod("getDesign", "unmarkedFramePCO",
                 removed.sites = out$removed.sites, go.dims = go.dims))
 })
 
+#Need to do this hacky approach because class union of PCO and MMO doesn't work
+#for reasons I don't understand
+setMethod("getDesign", "unmarkedFrameMMO",
+    function(umf, formula, na.rm=TRUE)
+{
+  class(umf)[1] <- "unmarkedFramePCO"
+  getDesign(umf, formula, na.rm)
+})
+
+# need a getDesign for distsampOpen.... not sure how to set this up
+# pcountOpenDS
+setMethod("getDesign", "unmarkedFrameDSO",
+    function(umf, formula, na.rm = TRUE)
+{
+    aschar1 <- as.character(formula)
+    aschar2 <- as.character(formula[[2]])
+    aschar3 <- as.character(formula[[2]][[2]])
+    aschar4 <- as.character(formula[[2]][[2]][[2]])
+
+    iotaformula <- as.formula(paste(aschar1[1], aschar1[3]))
+    pformula <- as.formula(paste(aschar2[1], aschar2[3]))
+    omformula <- as.formula(paste(aschar3[1], aschar3[3]))
+    gamformula <- as.formula(paste(aschar4[1], aschar4[3]))
+    lamformula <- as.formula(formula[[2]][[2]][[2]][[2]])
+
+    y <- getY(umf)
+    M <- nrow(y)
+    T <- umf@numPrimary
+    J <- ncol(y) / T
+    delta <- umf@primaryPeriod
+
+    if(is.null(umf@yearlySiteCovs))
+        yearlySiteCovs <- data.frame(placeHolder = rep(1, M*T))
+    else
+        yearlySiteCovs <- umf@yearlySiteCovs
+
+    ## add siteCovs in so they can be used as well
+    if(!is.null(umf@siteCovs)) {
+        sC <- umf@siteCovs[rep(1:M, each = T),,drop=FALSE]
+        yearlySiteCovs <- cbind(yearlySiteCovs, sC)
+        }
+
+    if(is.null(siteCovs(umf)))
+        siteCovs <- data.frame(placeHolder = rep(1, M))
+    else
+        siteCovs <- siteCovs(umf)
+
+    Xlam.mf <- model.frame(lamformula, siteCovs, na.action = stats::na.pass)
+    Xlam <- model.matrix(lamformula, Xlam.mf)
+    Xlam.offset <- as.vector(model.offset(Xlam.mf))
+    if(!is.null(Xlam.offset))
+        Xlam.offset[is.na(Xlam.offset)] <- 0
+
+    # Ignore last year of data
+    transCovs <- yearlySiteCovs[-seq(T, M*T, by=T),,drop=FALSE]
+    for(i in 1:ncol(transCovs))
+        if(is.factor(transCovs[,i]))
+            transCovs[,i] <- factor(transCovs[,i]) # drop unused levels
+
+    Xiota.mf <- model.frame(iotaformula, transCovs, na.action = stats::na.pass)
+    Xiota <- model.matrix(iotaformula, Xiota.mf)
+    Xiota.offset <- as.vector(model.offset(Xiota.mf))
+    if(!is.null(Xiota.offset))
+        Xiota.offset[is.na(Xiota.offset)] <- 0
+    
+    #Detection uses yearlySiteCovs
+    Xp.mf <- model.frame(pformula, yearlySiteCovs, na.action = stats::na.pass)
+    Xp <- model.matrix(pformula, Xp.mf)
+    Xp.offset <- as.vector(model.offset(Xp.mf))
+    if(!is.null(Xp.offset))
+        Xp.offset[is.na(Xp.offset)] <- 0
+    
+    Xgam.mf <- model.frame(gamformula, transCovs, na.action = stats::na.pass)
+    Xgam <- model.matrix(gamformula, Xgam.mf)
+    Xgam.offset <- as.vector(model.offset(Xgam.mf))
+    if(!is.null(Xgam.offset))
+        Xgam.offset[is.na(Xgam.offset)] <- 0
+    
+    Xom.mf <- model.frame(omformula, transCovs, na.action = stats::na.pass)
+    Xom <- model.matrix(omformula, Xom.mf)
+    Xom.offset <- as.vector(model.offset(Xom.mf))
+    if(!is.null(Xom.offset))
+        Xom.offset[is.na(Xom.offset)] <- 0
+
+    if(is.null(Xlam.offset)) Xlam.offset <- rep(0, M)
+    if(is.null(Xgam.offset)) Xgam.offset <- rep(0, M*(T-1))
+    if(is.null(Xom.offset)) Xom.offset <- rep(0, M*(T-1))
+    if(is.null(Xp.offset)) Xp.offset <- rep(0, M*T)
+    if(is.null(Xiota.offset)) Xiota.offset<- rep(0, M*(T-1))
+
+    # determine if gamma, omega, and iota are scalar, vector, or matrix valued
+    # Runtime is much faster for scalars and vectors
+    Xgo <- cbind(Xgam, Xom, Xiota)
+    getGOdims <- function(x) {
+        xm <- matrix(x, M, T-1, byrow=TRUE)
+#        anyNA <- apply(is.na(xm), 1, any)
+#        if(all(anyNA))
+#            return("matrix")
+#        xm <- xm[!anyNA,] # This is not 100% safe
+#        nSites <- nrow(xm)
+#        if(all(dim(unique(xm, MARGIN=1)) == c(1, T-1)))
+#            return("rowvec")
+#        else if(all(dim(unique(xm, MARGIN=2)) == c(nSites, 1)))
+#            return("colvec")
+#        else return("matrix")
+        col.table <- apply(xm, 2, table)
+        row.table <- apply(xm, 1, table)
+        if(is.vector(col.table) & !is.list(col.table)) {
+            return("rowvec")
+        } else if(is.vector(row.table) & !is.list(row.table)) {
+            return("colvec")
+        } else
+            return("matrix")
+        }
+    if(isTRUE(all.equal(gamformula,~1)) & isTRUE(all.equal(omformula, ~1)) &
+      isTRUE(all.equal(iotaformula, ~1)))
+        go.dims <- "scalar"
+    else {
+        go.dims.vec <- apply(Xgo, 2, getGOdims)
+        if(all(go.dims.vec == "rowvec"))
+            go.dims <- "rowvec"
+        else if(all(go.dims.vec == "colvec"))
+          ## NOTE: Temporary fix to the problem reported with 
+          ## time-only-varying covariates
+            go.dims <- "matrix" ##"colvec"  
+        else
+            go.dims <- "matrix"
+    }
+    
+    if(na.rm){
+      out <- handleNA(umf, Xlam, Xgam, Xom, Xp, Xiota,
+                      Xlam.offset, Xgam.offset, Xom.offset, Xp.offset, 
+                      Xiota.offset, delta)
+    } else {   
+      # delta needs to be formatted first
+      ya <- array(y, c(M, J, T))
+      yna <- apply(is.na(ya), c(1,3), all)
+      delta <- formatDelta(delta, yna)
+      out <- list(y=y, Xlam=Xlam, Xgam=Xgam, Xom=Xom, Xp=Xp, Xiota=Xiota,
+                  Xlam.offset=Xlam.offset, Xgam.offset=Xgam.offset,
+                  Xom.offset=Xom.offset, Xp.offset=Xp.offset,
+                  Xiota.offset=Xiota.offset,
+                  delta=delta, removed.sites=integer(0))
+    }
+
+    return(list(y = out$y, Xlam = out$Xlam, Xgam = out$Xgam,
+                Xom = out$Xom, Xp = out$Xp, Xiota = out$Xiota,
+                Xlam.offset=out$Xlam.offset, Xgam.offset=out$Xgam.offset,
+                Xom.offset=out$Xom.offset, Xp.offset=out$Xp.offset,
+                Xiota.offset=out$Xiota.offset, delta = out$delta,
+                removed.sites = out$removed.sites, go.dims = go.dims))
+})
 
 
 
@@ -1026,6 +1178,8 @@ setMethod("getDesign", "unmarkedFramePCO",
 
 
 
+
+#setMethod("handleNA", "unmarkedFramePCOorMMO",
 setMethod("handleNA", "unmarkedFramePCO",
     function(umf, Xlam, Xgam, Xom, Xp, Xiota, Xlam.offset, Xgam.offset,
              Xom.offset, Xp.offset, Xiota.offset, delta)
@@ -1037,12 +1191,13 @@ setMethod("handleNA", "unmarkedFramePCO",
 	T <- umf@numPrimary
 	y <- getY(umf)
 	J <- ncol(y) / T
+  R <- obsNum(umf)
 
 	Xlam.long <- Xlam[rep(1:M, each = J*T),]
 	Xlam.long.na <- is.na(Xlam.long)
 
 	long.na <- function(x) {
-            x.mat <- matrix(x, M, J*T, byrow = TRUE)
+            x.mat <- matrix(x, M, R, byrow = TRUE)
             x.mat <- is.na(x.mat)
             x.mat <- x.mat %*% obsToY
             x.long <- as.vector(t(x.mat))
@@ -1145,17 +1300,95 @@ setMethod("handleNA", "unmarkedFramePCO",
 })
 
 
+setMethod("handleNA", "unmarkedFrameDSO",
+    function(umf, Xlam, Xgam, Xom, Xp, Xiota, Xlam.offset, Xgam.offset,
+             Xom.offset, Xp.offset, Xiota.offset, delta)
+{
+	
+  y <- getY(umf)
+  M <- numSites(umf)
+  T <- umf@numPrimary
+  J <- ncol(y) / T
+  
+  ymat <- array(y, c(M,J,T))
 
+  #Apply NAs for entire observation if any intervals are NA
+  ymat <- array(y, c(M,J,T))
+  obs_any_na <- apply(ymat, c(1,3), function(x) any(is.na(x)))
+  any_na_ind <- which(obs_any_na, arr.ind=TRUE)
+  if(sum(obs_any_na)>0){
+    for(i in 1:nrow(any_na_ind)){
+      ymat[any_na_ind[i,1], ,any_na_ind[i,2]] <- NA
+    }
+  }
+  original_na_count <- sum(is.na(ymat))
 
+  #NAs in lambda design matrix - have to delete entire site
+  site_NA <- apply(Xlam, 1, function(x) any(is.na(x)))
+  ymat[site_NA,,] <- NA
+  
+  find_na <- function(ymat, dm, M, T){
+    dm <- matrix(apply(dm, 1, function(x) any(is.na(x))), M, T, byrow=TRUE)
+    na_ind <- which(dm, arr.ind=TRUE)
+    if(sum(dm)>0){
+      for (i in 1:nrow(na_ind)){
+        ymat[na_ind[i,1], ,na_ind[i,2]] <- NA
+      }
+    }
+    ymat
+  }
 
+  #NAs in other design mats
+  ymat <- find_na(ymat, Xgam, M, T-1)
+  ymat <- find_na(ymat, Xom, M, T-1)
+  ymat <- find_na(ymat, Xiota, M, T-1)
+  ymat <- find_na(ymat, Xp, M, T)
+  
+  if(sum(is.na(ymat)) > original_na_count){
+    warning("Some observations have been discarded because corresponding covariates were missing.", call. = FALSE)
+  }
 
+  #Format delta
+  ytna <- apply(ymat, c(1,3), function(x) all(is.na(x)))
+	delta <- formatDelta(delta, ytna)
 
+  #Reformat y
+  y <- matrix(ymat, nrow=M)
 
+  #Remove sites
+  sites.to.remove <- which(apply(ymat, 1, function(x) all(is.na(x))))
+  nrem <- length(sites.to.remove)
+  if(nrem > 0){
+    y <- y[-sites.to.remove,]
+    Xlam <- Xlam[-sites.to.remove,,drop=FALSE]
+    Xlam.offset <- Xlam.offset[-sites.to.remove]
+    delta <- delta[-sites.to.remove,,drop=FALSE]
+
+    ysc_rem <- rep(1:M, each=(T-1)) %in% sites.to.remove 
+    Xgam <- Xgam[!ysc_rem,, drop=FALSE]
+    Xgam.offset <- Xgam.offset[!ysc_rem]
+    Xom <- Xom[!ysc_rem,, drop=FALSE]
+    Xom.offset <- Xom.offset[!ysc_rem]
+    Xiota <- Xiota[!ysc_rem,, drop=FALSE]
+    Xiota.offset <- Xiota.offset[!ysc_rem]
+    
+    p_rem <- rep(1:M, each=T) %in% sites.to.remove
+    Xp <- Xp[!p_rem,, drop=FALSE]
+    Xp.offset <- Xp.offset[!p_rem]
+
+    warning(paste(nrem, "sites have been discarded because of missing data."), call.=FALSE)
+  }
+
+	list(y = y, Xlam = Xlam, Xgam = Xgam, Xom = Xom, Xp = Xp, Xiota = Xiota,
+             Xlam.offset=Xlam.offset, Xgam.offset=Xgam.offset,
+             Xom.offset=Xom.offset, Xp.offset=Xp.offset,
+             Xiota.offset=Xiota.offset, delta = delta,
+             removed.sites = sites.to.remove)
+})
 
 
 # UnmarkedFrameGMN
 
-
 setMethod("getDesign", "unmarkedFrameG3",
     function(umf, formula, na.rm = TRUE)
 {
