@@ -153,15 +153,7 @@ if(engine=="R"){
 
 if(missing(starts)) starts <- rep(0, nP)
 fm <- optim(starts, nll, method = method, hessian = se, ...)
-opt <- fm
-if(se) {
-    covMat <- tryCatch(solve(fm$hessian), error=function(x)
-        simpleError("Hessian is singular. Try using fewer covariates."))
-    if(identical(class(covMat)[1], "simpleError")) {
-        warning(covMat$message)
-        covMat <- matrix(NA, nP, nP)
-        }
-    } else covMat <- matrix(NA, nP, nP)
+covMat <- invertHessian(fm, nP, se)
 ests <- fm$par
 fmAIC <- 2 * fm$value + 2 * nP
 
@@ -203,7 +195,7 @@ if(identical(mixture,"NB"))
 umfit <- new("unmarkedFitGMM", fitType = "gmn",
     call = match.call(), formula = form, formlist = formlist,
     data = data, estimates = estimateList, sitesRemoved = D$removed.sites,
-    AIC = fmAIC, opt = opt, negLogLike = fm$value, nllFun = nll,
+    AIC = fmAIC, opt = fm, negLogLike = fm$value, nllFun = nll,
     mixture=mixture, K=K)
 
 return(umfit)
