@@ -81,13 +81,7 @@ occu <- function(formula, data, knownOcc = numeric(0),
 
     if(missing(starts)) starts <- rep(0, nP)
     fm <- optim(starts, nll, method = method, hessian = se, ...)
-    opt <- fm
-    if(se) {
-        tryCatch(covMat <- solve(fm$hessian),
-                 error=function(x) stop(simpleError("Hessian is singular.  Try providing starting values or using fewer covariates.")))
-    } else {
-        covMat <- matrix(NA, nP, nP)
-    }
+    covMat <- invertHessian(fm, nP, se)
     ests <- fm$par
     fmAIC <- 2 * fm$value + 2 * nP #+ 2*nP*(nP + 1)/(M - nP - 1)
     names(ests) <- c(occParms, detParms)
@@ -110,7 +104,7 @@ occu <- function(formula, data, knownOcc = numeric(0),
     umfit <- new("unmarkedFitOccu", fitType = "occu", call = match.call(),
                  formula = formula, data = data,
                  sitesRemoved = designMats$removed.sites,
-                 estimates = estimateList, AIC = fmAIC, opt = opt,
+                 estimates = estimateList, AIC = fmAIC, opt = fm,
                  negLogLike = fm$value,
                  nllFun = nll, knownOcc = knownOccLog)
 
